@@ -1,7 +1,7 @@
 ### DPsurvint.R                    
 ### Fit a semiparametric aft model for interval censored data.
 ###
-### Copyright: Alejandro Jara, 2006-2009.
+### Copyright: Alejandro Jara, 2006-2010.
 ###
 ### Last modification: 22-12-2006.
 ###
@@ -401,7 +401,7 @@ function(object,grid,xnew=NULL,hpd=TRUE, ...)
    	     nsave<-dim(object$save.state$thetasave)[1]
    	     mulog<-object$save.state$thetasave[,(object$p+1)]
 	     sdlog<-sqrt(object$save.state$thetasave[,(object$p+2)])
-	     alpha<-sqrt(object$save.state$thetasave[,(object$p+4)])
+	     alpha<-object$save.state$thetasave[,(object$p+4)]
 	      
 	     v<-matrix(object$save.state$randsave,nsave,nrec+1)
 	     vpred<-v[,(nrec+1)]
@@ -420,11 +420,11 @@ function(object,grid,xnew=NULL,hpd=TRUE, ...)
    	     
    	     mulog<-object$save.state$thetasave[,(object$p+1)]
    	     sdlog<-sqrt(object$save.state$thetasave[,(object$p+2)])
-   	     alpha<-sqrt(object$save.state$thetasave[,(object$p+4)])
+   	     alpha<- object$save.state$thetasave[,(object$p+4)]
    	     
-   	     v<-matrix(object$save.state$randsave,nsave,nrec+1)
-   	     vpred<-v[,(nrec+1)]
-   	     v<-v[,1:nrec]
+   	     v <- matrix(object$save.state$randsave,nsave,nrec+1)
+   	     vpred <- v[,(nrec+1)]
+   	     v <- v[,1:nrec]
 
    	     if (object$p != pnew) 
    	     {
@@ -432,20 +432,21 @@ function(object,grid,xnew=NULL,hpd=TRUE, ...)
 	        in the original model.\n")
 	     }
 
-	     covn<-rep(0,npred) 
+	     covn <- rep(0,npred) 
 	     
 	     for(i in 1:npred)
 	     {
-                 covnw<-round(xnew[i,1],3)
+			 covnw<-round(xnew[i,1],3)
                  
-                 if(pnew>1)
-                 {
-                   for(j in 2:pnew){
-                       covnw<-paste(covnw,round(xnew[i,j],3),sep=";") 
-                   }
-                 }  
-                 covn[i]<-covnw  
-             }    
+			 if(pnew>1)
+			 {
+				for(j in 2:pnew)
+				{
+					covnw<-paste(covnw,round(xnew[i,j],3),sep=";") 
+				}
+			}  
+			covn[i]<-covnw  
+	 	 }    
 	 }    
 
  	 pm<-matrix(0,npred,ngrid)
@@ -458,59 +459,80 @@ function(object,grid,xnew=NULL,hpd=TRUE, ...)
 	 expxb<-exp(xnew%*%t(object$save.state$thetasave[,1:object$p]))
 	        
 	 for(i in 1:npred)
-         {
-	     surv<-matrix(0,nrow=ngrid,ncol=nsave)
+	 {
+	     surv <- matrix(0,nrow=ngrid,ncol=nsave)
 	     for(j in 1:ngrid)
 	     {
 	         for(k in 1:nsave)
 	         {
-			linf<-grid[j]*expxb[i,k]
-			mwork<-mulog[k]
-			swork<-sdlog[k]
-			awork<-alpha[k]
-			vwork<-v[k,]
-			Cdelta<- sum(vwork>linf)
-			Cparam<- awork*plnorm(linf,meanlog=mwork,sdlog=swork,
-			                      lower.tail=FALSE,log.p=FALSE)
-			tmp1<-(Cparam+Cdelta)
-			tmp2<-alpha[k]+nrec-tmp1
+				 linf <- grid[j]*expxb[i,k]
+				 mwork <- mulog[k]
+				 swork <- sdlog[k]
+				 awork <- alpha[k]
+				 vwork <- v[k,]
+				 Cdelta <- sum(vwork>linf)
+				 Cparam <- awork*plnorm(linf,meanlog=mwork,sdlog=swork,
+						  			    lower.tail=FALSE,log.p=FALSE)
+				 
+				 if(is.null(Cparam))
+				 { 
+					 cat(Cparam)
+					 stop("Error")
+				 }
+				 if(is.null(Cdelta))
+				 {
+				     cat(Cdelta)
+					 stop("Error") 
+                 }    
+				 
+				 tmp1 <- (Cparam+Cdelta)
+				 tmp2 <- alpha[k]+nrec-tmp1
 
-			if(tmp2==0)
-			{
-			  surv[j,k]<-1
-			}
-			if(tmp1==0)
-			{
-			  surv[j,k]<-0
-			}
-			if(tmp1 != 0 && tmp2 != 0)
-			{
-			  surv[j,k]<-rbeta(n=1,shape1=tmp1,shape2=tmp2)
-			}  
-	          }
-	          pm[i,j]<-mean(surv[j,])
-		  pmed[i,j]<-median(surv[j,])
-		  psd[i,j]<-sqrt(var(surv[j,]))
-		  pstd[i,j]<-sqrt(var(surv[j,]))/sqrt(nsave)
+				 tmp3 <- tmp1/(tmp1+tmp2)
+				 if(tmp2<=0.1)
+				 {
+					 tmp3 <- 1
+				 }
+				 if(tmp1<=0.1)
+				 {
+					 tmp3 <- 0
+				 }
+				 if((tmp1 > 0.1) & (tmp2 > 0.1))
+				 {
+					 tmp3 <- rbeta(1,tmp1,tmp2) 
+				 }
+				 surv[j,k] <- tmp3
+				 if(is.na(surv[j,k]))
+				 {
+				     cat(tmp1,"\n")
+					 cat(tmp2,"\n")
+					 stop("Error")
+				 }	 
+			 }
+			 
+			 pm[i,j]<-mean(surv[j,])
+			 pmed[i,j]<-median(surv[j,])
+			 psd[i,j]<-sqrt(var(surv[j,]))
+			 pstd[i,j]<-sqrt(var(surv[j,]))/sqrt(nsave)
 		  
-                  if(hpd==TRUE)
-                  {
-                        alow<-rep(0,2)
-                        aupp<-rep(0,2)
-                        sig<-0.05
-                        vec<-surv[j,]
-                        n<-length(vec)
-                        a<-.Fortran("hpd",n=as.integer(n),alpha=as.double(sig),x=as.double(vec),
-                                     alow=as.double(alow),aupp=as.double(aupp),PACKAGE="DPpackage")
-                        plinf[i,j]<-a$alow[1]            
-                        plsup[i,j]<-a$aupp[1]            
-                  }
-                  else
-                  {
+			if(hpd==TRUE)
+			{
+				alow<-rep(0,2)
+				aupp<-rep(0,2)
+				sig<-0.05
+				vec<-surv[j,]
+				n<-length(vec)
+				a<-.Fortran("hpd",n=as.integer(n),alpha=as.double(sig),x=as.double(vec),
+							alow=as.double(alow),aupp=as.double(aupp),PACKAGE="DPpackage")
+				plinf[i,j]<-a$alow[1]            
+				plsup[i,j]<-a$aupp[1]            
+			}
+			else
+			{
 		        plinf[i,j]<-quantile(surv[j,],0.025)		
 		        plsup[i,j]<-quantile(surv[j,],0.975)			                
-                  }
-	      }
+			}
+		}
    	 }
    	 
    	 dimnames(pm)<-list(covn,grid)
